# Resampling (and mixing) of life stages to make genetic inferences

# Load packages
require(adegenet)
require(hierfstat)
require(plyr)
library(reshape)
library(ecodist)

# Read in data files
sibs <- read.fstat("AMAC_Sibs.dat")
n.sibs <- read.fstat("AMAC_NoSibs.dat")
dist <- read.csv("Pond_Dist.csv")[,3]

# Create bootstrap function
iters <- 1000
ar <- matrix(nrow=iters,ncol=5)
AR <- list(ar,ar)
fst<-matrix(nrow=10,ncol=iters)
FST<-list(fst,fst)
Dc <- list(fst,fst)
POPS<-c(0,25,50,75,100)
het<-matrix(nrow=iters,ncol=5)
HET<-list(het,het)
for (i in 1: iters){
# Subsample Fstat data object
sib.sub<-subsampind(dat=sibs,sampsize=25)
n.sib.sub<-subsampind(dat=n.sibs,sampsize=25)

# Bootstrap allelic richness estimates for each life stage
ar.sibs<-allelic.richness(data=sib.sub,min.n=34)$Ar

ar.n.sibs<-allelic.richness(data=n.sib.sub,min.n=34)$Ar

ar.obs<-list(ar.sibs,ar.n.sibs)

ar.mean <- lapply(X=ar.obs,colMeans)

for(j in 1:length(ar.obs)){
AR[[j]][i,]<-ar.mean[[j]]
  }

# Calculate Fst
Fst <- pp.fst(sib.sub)$fst.pp
diag(Fst) <- 0
FST[[1]][,i] <- lower(t(Fst))

Fst <- pp.fst(n.sib.sub)$fst.pp
diag(Fst) <- 0
FST[[2]][,i] <- lower(t(Fst))

# Make genind object 
sib.sub1 <- df2genind(sib.sub[,2:16],pop=as.factor(sib.sub$Pop), ncode = 3)
sib.sub <- genind2genpop(sib.sub1,other.action = "NA")
sib.sub <- as(sib.sub,'genpop')

Dc[[1]][,i] <- lower(dist.genpop(sib.sub,method = 2,upper = T)) 

n.sib.sub1 <- df2genind(n.sib.sub[,2:16],pop=as.factor(n.sib.sub$Pop), ncode = 3)
n.sib.sub <- genind2genpop(n.sib.sub1,other.action = "NA")
n.sib.sub <- as(n.sib.sub,'genpop')

Dc[[2]][,i] <- lower(dist.genpop(n.sib.sub,method = 2,upper = T)) 

for (k in 1:length(POPS)){
  l=POPS[k]
  HET[[1]][i,k]<-mean(summary(sib.sub1[(l+1):(l+25),])$Hobs)
  HET[[2]][i,k]<-mean(summary(n.sib.sub1[(l+1):(l+25),])$Hobs)
    }
}

AR.mean<-lapply(AR,colMeans)
AR.CI<-lapply(AR,function(x) return(apply(x,2,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))))
AR.sib<-cbind(as.matrix(AR.mean[[1]]),as.matrix(AR.CI[[1]][1,]),as.matrix(AR.CI[[1]][2,]))
colnames(AR.sib)<-c("mean","2.5%","97.5%")
AR.n.sib<-cbind(as.matrix(AR.mean[[2]]),as.matrix(AR.CI[[2]][1,]),as.matrix(AR.CI[[2]][2,]))
colnames(AR.n.sib)<-c("mean","2.5%","97.5%")
AR.g.mean <- lapply(AR,mean)
AR.g.CI<-lapply(AR,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
AR.g.mean.sib<-c(AR.g.mean[[1]],AR.g.CI[[1]][1],AR.g.CI[[1]][2])
AR.g.mean.n.sib<-c(AR.g.mean[[2]],AR.g.CI[[2]][1],AR.g.CI[[2]][2])


Fst.mean<-lapply(FST,rowMeans)
Fst.CI<-lapply(FST,function(x) return(apply(x,1,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))))
Fst.sib<-cbind(as.matrix(Fst.mean[[1]]),as.matrix(Fst.CI[[1]][1,]),as.matrix(Fst.CI[[1]][2,]))
colnames(Fst.sib)<-c("mean","2.5%","97.5%")
Fst.n.sib<-cbind(as.matrix(Fst.mean[[2]]),as.matrix(Fst.CI[[2]][1,]),as.matrix(Fst.CI[[2]][2,]))
colnames(Fst.n.sib)<-c("mean","2.5%","97.5%")
Fst.g.mean <- lapply(FST,mean)
Fst.g.CI<-lapply(FST,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
Fst.g.mean.sib<-c(Fst.g.mean[[1]],Fst.g.CI[[1]][1],Fst.g.CI[[1]][2])
Fst.g.mean.n.sib<-c(Fst.g.mean[[2]],Fst.g.CI[[2]][1],Fst.g.CI[[2]][2])
Mantelp.Fst<-lapply(FST,function(x) return(apply(x,2,function(x) return(mantel(x~dist,nperm=100000)[2]))))
Mantelr.Fst<-lapply(FST,function(x) return(apply(x,2,function(x) return(mantel(x~dist,nperm=1)[1]))))
Mantelp.Fst.mean<-lapply(Mantelp.Fst,mean)
Mantelr.Fst.mean<-lapply(Mantelr.Fst,mean)
Mantelp.Fst.CI<-lapply(Mantelp.Fst,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
Mantelr.Fst.CI<-lapply(Mantelr.Fst,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
Mantel.Results.sib.Fst<-c(Mantelr.Fst.mean[[1]], Mantelr.Fst.CI[[1]][1], Mantelr.Fst.CI[[1]][2],Mantelp.Fst.mean[[1]],Mantelp.Fst.CI[[1]][1],Mantelp.Fst.CI[[1]][2])
Mantel.Results.n.sib.Fst<-c(Mantelr.Fst.mean[[2]], Mantelr.Fst.CI[[2]][1], Mantelr.Fst.CI[[2]][2],Mantelp.Fst.mean[[2]],Mantelp.Fst.CI[[2]][1],Mantelp.Fst.CI[[1]][2])
names(Mantel.Results.sib.Fst)<-c("Mantelr.Fst.mean","2.5%", "97.5%","pval","2.5%", "97.5%")
names(Mantel.Results.n.sib.Fst)<-c("Mantelr.Fst.mean","2.5%", "97.5%","pval","2.5%", "97.5%")
# Proportion of bootstraps significant (i.e. error rate)
(mantel.error.Fst <- lapply(Mantelp.Fst,function(x) return((ecdf(x)(0.05)))))

## Caclulate summaries with Dc
Dc.mean<-lapply(Dc,rowMeans)
Dc.CI<-lapply(Dc,function(x) return(apply(x,1,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))))
Dc.sib<-cbind(as.matrix(Dc.mean[[1]]),as.matrix(Dc.CI[[1]][1,]),as.matrix(Dc.CI[[1]][2,]))
colnames(Dc.sib)<-c("mean","2.5%","97.5%")
Dc.n.sib<-cbind(as.matrix(Dc.mean[[2]]),as.matrix(Dc.CI[[2]][1,]),as.matrix(Dc.CI[[2]][2,]))
colnames(Dc.n.sib)<-c("mean","2.5%","97.5%")
Dc.g.mean <- lapply(Dc,mean)
Dc.g.CI<-lapply(Dc,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
Dc.g.mean.sib<-c(Dc.g.mean[[1]],Dc.g.CI[[1]][1],Dc.g.CI[[1]][2])
Dc.g.mean.n.sib<-c(Dc.g.mean[[2]],Dc.g.CI[[2]][1],Dc.g.CI[[2]][2])
Mantelp.Dc<-lapply(Dc,function(x) return(apply(x,2,function(x) return(mantel(x~dist,nperm=100000)[2]))))
Mantelr.Dc<-lapply(Dc,function(x) return(apply(x,2,function(x) return(mantel(x~dist,nperm=1)[1]))))
Mantelp.Dc.mean<-lapply(Mantelp.Dc,mean)
Mantelr.Dc.mean<-lapply(Mantelr.Dc,mean)
Mantelp.Dc.CI<-lapply(Mantelp.Dc,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
Mantelr.Dc.CI<-lapply(Mantelr.Dc,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
Mantel.Results.sib.Dc<-c(Mantelr.Dc.mean[[1]], Mantelr.Dc.CI[[1]][1], Mantelr.Dc.CI[[1]][2],Mantelp.Dc.mean[[1]],Mantelp.Dc.CI[[1]][1],Mantelp.Dc.CI[[1]][2])
Mantel.Results.n.sib.Dc<-c(Mantelr.Dc.mean[[2]], Mantelr.Dc.CI[[2]][1], Mantelr.Dc.CI[[2]][2],Mantelp.Dc.mean[[2]],Mantelp.Dc.CI[[2]][1],Mantelp.Dc.CI[[1]][2])
names(Mantel.Results.sib.Dc)<-c("Mantelr.Dc.mean","2.5%", "97.5%","pval","2.5%", "97.5%")
names(Mantel.Results.n.sib.Dc)<-c("Mantelr.Dc.mean","2.5%", "97.5%","pval","2.5%", "97.5%")
# Proportion of bootstraps significant (i.e. error rate)
(mantel.error.Dc <- lapply(Mantelp.Dc,function(x) return((ecdf(x)(0.05)))))


Het.mean<-lapply(HET,colMeans)
Het.CI<-lapply(HET,function(x) return(apply(x,2,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))))
Het.sib<-cbind(as.matrix(Het.mean[[1]]),as.matrix(Het.CI[[1]][1,]),as.matrix(Het.CI[[1]][2,]))
colnames(Het.sib)<-c("mean","2.5%","97.5%")
Het.n.sib<-cbind(as.matrix(Het.mean[[2]]),as.matrix(Het.CI[[2]][1,]),as.matrix(Het.CI[[2]][2,]))
colnames(Het.n.sib)<-c("mean","2.5%","97.5%")
Het.g.mean <- lapply(HET,mean)
Het.g.CI<-lapply(HET,function(x) return(quantile(x=x,probs=c(.025,.975),na.rm=T,type=c(8))))
Het.g.mean.sib<-c(Het.g.mean[[1]],Het.g.CI[[1]][1],Het.g.CI[[1]][2])
Het.g.mean.n.sib<-c(Het.g.mean[[2]],Het.g.CI[[2]][1],Het.g.CI[[2]][2])

# Write boot results to file
write.table(x=AR.sib,file="AR.Sibs_Boot_new.csv",sep=",",col.names=T,row.names=F)
write.table(x=AR.n.sib,file="AR.NoSibs_Boot_new.csv",sep=",",col.names=T,row.names=F)
write.table(x=Fst.sib,file="Fst.Sibs_Boot_new.csv",sep=",",col.names=T,row.names=F)
write.table(x=Fst.n.sib,file="Fst.NoSibs_Boot_new.csv",sep=",",col.names=T,row.names=F)
write.table(x=Dc.sib,file="Dc.Sibs_Boot_new.csv",sep=",",col.names=T,row.names=F)
write.table(x=Dc.n.sib,file="Dc.NoSibs_Boot_new.csv",sep=",",col.names=T,row.names=F)
write.table(x=Het.sib,file="Het.Sibs_Boot_new.csv",sep=",",col.names=T,row.names=F)
write.table(x=Het.n.sib,file="Het.NoSibs_Boot_new.csv",sep=",",col.names=T,row.names=F)


